@@ -1,17 +1,17 @@
 import itertools
-import os
 from dataclasses import dataclass
 from datetime import date
 from pathlib import Path
-from typing import Iterator, NoReturn, Optional
+from typing import NoReturn
 
-from dotenv import find_dotenv, load_dotenv
-from lawsql_utils.trees import data_tree_walker
+import yaml
+from citation_decision import CitationDocument
+from dateutil.parser import parse
+from slugify import slugify
+from statute_matcher import StatuteMatcher
+from treeish import fetch_values_from_key, set_node_ids
 
-from .fts import get_rows_with_key_from_dict
-from .utils import format_citations, get_citation, set_tree_ids
-
-load_dotenv(find_dotenv())
+from .statute_formatter import format_units
 
 
 @dataclass
@@ -36,10 +36,8 @@ class CodificationItem:
     path_to_code_yaml: Path
 
     def __post_init__(self):
-        from lawsql_utils.general import format_full_date
-
         self.data: dict = self.fetch_complete_data_from_file
-        self.publication_date: date = format_full_date(self.data["date"])
+        self.publication_date: date = parse(self.data["date"]).date()
         self.sync = self.statutes_in_sync
         self.units: list[dict] = self.data["units"]
         self.version = str(self.data["version"])
@@ -52,10 +50,8 @@ class CodificationItem:
     @property
     def fetch_complete_data_from_file(self) -> NoReturn | dict:
         """If there are any missing keys, raise error."""
-        from lawsql_tree_unit import format_units
-        from lawsql_utils.files import load_yaml_from_path
-
-        raw_data = load_yaml_from_path(self.path_to_code_yaml)
+        text = self.path_to_code_yaml.read_text()
+        raw_data = yaml.load(text, Loader=yaml.SafeLoader)
         for key in [
             "title",
             "description",
@@ -70,45 +66,34 @@ class CodificationItem:
             if key not in raw_data:
                 raise Exception(f"Missing {key}")
         format_units(raw_data["units"])  # fixes item, caption, content
-        format_citations(raw_data["units"])  # adjusts the canonical citations
-        set_tree_ids(raw_data["units"])  # adds id to each node
+        self.format_citations(
+            raw_data["units"]
+        )  # adjusts the canonical citations
+        set_node_ids(raw_data["units"])  # adds id to each node
 
         return raw_data
 
-    @property
-    def decision_fts_row_ids(self) -> Iterator[int]:
-        """Each of the units may contain query strings under a `queries` key. Collect all of these from all the units to get the rowids of the Decisions table."""
-        source = {"units": self.units}
-        rows = get_rows_with_key_from_dict(source, "queries")
-        for data in rows:
-            yield data["rowid"]
-
-    @property
-    def citations_from_histories(self) -> set[str] | NoReturn:
-        """Get all strings of Decision Citations found in the "citation" key in the nested dictionary"""
-        return set(data_tree_walker(self.data, "citation"))
-
-    @property
-    def citations_in_db(self) -> list[str] | NoReturn:
-        """If no error is raised, either there are no citations (an empty list) in the codification or all citations have a corresponding match (a list of dicts)"""
-        matched_citations = []
-        if texts := self.citations_from_histories:
-            citation_list = list(texts)  # convert set to list
-            for text in citation_list:
-                if citation_found := get_citation(text):
-                    matched_citations.append(citation_found["raw"].canonical)
-                else:
-                    err_msg = f"Could not find citation in index.db database: {text=}"
-                    raise Exception(err_msg)
-        return matched_citations
+    def format_citations(self, nodes: list[dict]) -> None:
+        for node in nodes:
+            if histories := node.get("history", None):
+                for history in histories:
+                    if history.get("citation", None):
+                        doc = CitationDocument(history["citation"])
+                        history["citation"] = doc.first_canonical
+            if new_nodes := node.get("units", None):
+                self.format_citations(new_nodes)  # call self
 
     @property
     def statutes_from_histories(self) -> set[str] | NoReturn:
         """Get all strings of Statutes found in the "statute" key in the nested dictionary"""
-        if not (blocks := set(data_tree_walker(self.data, "statute"))):
+        if not (blocks := set(fetch_values_from_key(self.data, "statute"))):
             raise Exception(f"No statute blocks found; see {blocks=}")
         return blocks
 
+    @property
+    def decisions_from_citations(self) -> list[str]:
+        return list(set(fetch_values_from_key(self.data, "citation")))
+
     @property
     def requisites_sans_base(self) -> set[str]:
         """Get set of texts representing statute serial names without the base statute for the codification."""
@@ -131,7 +116,7 @@ class CodificationItem:
     @property
     def get_history_lists(self) -> list[list[dict]] | NoReturn:
         """Each unit may have its history; each history is a list of blocks"""
-        if not (l := list(data_tree_walker(self.data, "history"))):
+        if not (l := list(fetch_values_from_key(self.data, "history"))):
             raise Exception(f"No history blocks; see {l=}")
         return l
 
@@ -140,26 +125,13 @@ class CodificationItem:
         """Get histories by flattening list (chain)"""
         return list(itertools.chain(*self.get_history_lists))
 
-
-def get_matching_yaml_filenames(
-    folder: Path, q: Optional[str] = None
-) -> Iterator[Path]:
-    """With folder given, return generator of yaml files; if a specific string `q` is supplied, only match proper files."""
-    return folder.glob(f"{q}*.yaml" if q else "*.yaml")
-
-
-def get_author_codification_folder(
-    author: str = "mv3",
-) -> NoReturn | Path:
-    """Implies preset `PATH_TO_LAWSQL` with subfolder `codifications`"""
-    l = os.getenv("PATH_TO_LAWSQL").split("/") + ["codifications", author]
-    folder = Path.home().joinpath(*l)
-    if not folder.exists():
-        raise Exception(f"{folder=} could not be found.")
-    return folder
-
-
-def get_codifications(author: str = "mv3", q: str = None):
-    """Gets all codifications within a specific author `folder` (default is `mv3`), matching the query `q` of a preset base `PATH_TO_LAWSQL` with subfolder `codifications`"""
-    folder = get_author_codification_folder(author)
-    return get_matching_yaml_filenames(folder, q)
+    @property
+    def slug(self) -> str:
+        matched = StatuteMatcher(self.base).matches[0]
+        cat = matched.category
+        idx = matched.identifier
+        year = self.publication_date.year
+        version = f"v{self.version}"
+        folder = self.path_to_code_yaml.parent.stem
+        slug = slugify(f"{cat}-{idx}-pubyear-{year}-{version}-by-{folder}")
+        return slug
